<?php


namespace App\Helpers;

use aosi\Aosi_core;
use aosi\Config;
use aosi\HrEduOrg;
use aosi\HrEduPerson;
use aosi\Logger;
use App\Extensions\LdapUser;
use Illuminate\Support\Facades\Auth;

class AosiCoreHelper
{
    private static $aosiCore;

    private static $baseDN;

    private static $isSingleRealmInstance;

    protected static $requirePasswordChangeKey = 'requirePasswordChange';

    protected static $adminTypes = [
        1 => 'Administator definiran u konfiguracijskoj datoteci',
        2 => 'Administator definiran u imeniku',
        3 => 'Administrator iz imenika s ograničenjem',
    ];

    /**
     * Atributi koji se koriste za generiranje administratorskih ograničenja unutar hrEduPersonRole
     * @var array
     */
    const ADMIN_LIMIT_ATTRIBUTES = ['ou', 'hredupersonaffiliation'];

    /**
     * Vrijednost atributa hrEduPersonRole koja označava da korisnik ima pravo administriranja imenika
     * @var string
     */
    const ADMIN_VALUE_DESIGNATOR = 'administrator imenika';

    /**
     * Ključ u polju atributa koje vraća metoda userSearch() metoda iz AOSI Core-a, a koji indicira da je aktivan
     * password policy za pojedinog korisnika
     * @var string
     */
    const PASSWORD_POLICY_INDICATOR_KEY = 'pwdpolicysubentry';

    /**
     * Atributi čija vrijednost ima prefix i za koje je onda potreban poseban hendling.
     * ['ključ atributa' => 'ključ propertija koji sadrži vrijednosti prefixa u definiciji atributa']
     * @var array
     */
    protected static $prefixedAttributes = [
        'hredupersonuniquenumber' => 'hredupersonuniquenumber_prefix',
        'hreduorguniquenumber' => 'hreduorguniquenumber_prefix',
    ];

    /**
     * Default date format koji se prikazuje korisniku
     * @var string
     */
    protected static $defaultDateFormat = 'd.m.Y.';

    /**
     * Onemogućeno kreiranje istance kroz konstruktor zbog vraćanja jedne instance.
     * AosiCoreHelper constructor.
     */
    private function __construct() {}

    /**
     * Kreiraj novu ili vrati postojeću instancu AOSI Core-a.
     *
     * @return Aosi_core
     */
    public static function init()
    {
        if (!static::$aosiCore) {
            Config::load();
            Logger::prepare();
            ini_set('date.timezone',Config::get('timezone'));

            HrEduPerson::l();
            HrEduOrg::l();

            static::$aosiCore = new Aosi_core();
        }

        return static::$aosiCore;
    }

    /**
     * Dohvati listu atributa koji imaju prefixe u vrijednostima
     * @return array
     */
    public static function getPrefixedAttributes()
    {
        return static::$prefixedAttributes;
    }

    /**
     * Vrati base DN trenutno autenticiranog korisnika, ili null ako korisnik nije autenticiran.
     * @return string|null
     */
    public static function getBaseDn()
    {
        // Nakon autentikacije korisnika, u sesiji ostaje zapamćeno na koji realm je obavljena sama autentikacija,
        // pa to možemo iskoristiti...
        if (Auth::user()) {
            return Auth::user()->basedn[0] ?? null;
        }

        return null;
    }

    /**
     * Generiraj base DN obzirom na to dali je aplikacija zadužena samo za jedan realm (default iz konfiguracije),
     * ili za više realm-ova (generiraj za dani realm).
     *
     * @param string|null $realm
     * @return string
     */
    public static function generateBaseDn($realm = null)
    {
        if (AosiCoreHelper::isSingleRealmInstance()) {
            return static::getDefaultBaseDn();
        } else {
            return AosiCoreHelper::generateBaseDnForRealm($realm);
        }
    }

    /**
     * Provjeri dali se radi o instanci aplikacije koja je zadužena za samo jedan realm (konfigurabilno u aosi_config)
     * @return mixed
     */
    public static function isSingleRealmInstance()
    {
        return Config::get('is_single_realm_instance');
    }

    /**
     * Varti defaultni base DN
     * @return mixed
     */
    public static function getDefaultBaseDn()
    {
        $defaultRealm = static::getDefaultRealm();

        return static::generateBaseDnForRealm($defaultRealm);
    }

    /**
     * Provjeri dali se radi o instanci aplikacije koja je zadužena za samo jedan realm (konfigurabilno u aosi_config)
     * @return mixed
     */
    public static function getDefaultRealm()
    {
        // Defaultni realm je prvi realm iz konfiguracije.
        return Config::get('realm')[0] ?? null;
    }

    public static function getRealms()
    {
        // Ako je aplikacija zadužena za samo jedan realm, vrati samo prvi realm iz konfiguracije (makar ih bilo više).
        if (static::isSingleRealmInstance()) {
            return [static::getDefaultRealm()];
        }

        // Vrati sve realm-ove iz konfiguracije.
        return Config::get('realm');
    }

    public static function generateBaseDnForRealm($realm)
    {
        if (!$realm) {
            throw new \InvalidArgumentException('Nije moguće generirati Base DN za parazan realm.');
        }

        if(!in_array($realm, Config::get('realm'))) {
            throw new \InvalidArgumentException('Aplikacija nije konfigurirana za dani realm.');
        }

        return 'dc=' . $realm . ',dc=hr';
    }

    public static function generateLdapFilter(array $filters, $operator = '&')
    {
        if (!preg_match('/^[&|]$/', $operator)) {
            throw new \Exception('Dani operator (trenutno) nije podržan.');
        }

        $ldapFilter = '(' . $operator;

        foreach ($filters as $key => $value) {
            $ldapFilter .= '(' . $key . '=' . $value . ')';
        }

        $ldapFilter .= ')';

        return $ldapFilter;
    }

    /**
     * Postavi ili makni naznaku za promjenom passworda za korisnika
     *
     * @param bool $shouldChangePassword
     */
    public static function setPasswordChangeRequirement($shouldChangePassword = true)
    {
        if ($shouldChangePassword) {
            static::notifyUserAboutPasswordChangeRequirement();
            session()->put(self::$requirePasswordChangeKey, true);
        } else {
            session()->forget(self::$requirePasswordChangeKey);
        }
    }

    /**
     * Dali korisnik mora promijeniti lozinku
     * @return bool
     */
    public static function isPasswordChangeRequired()
    {
        return session()->has(self::$requirePasswordChangeKey);
    }

    public static function notifyUserAboutPasswordChangeRequirement()
    {
        $message = 'Prije nego nastavite s korištenjem aplikacije, molimo promijenite vašu zaporku';
        flash($message)->warning()->important();
    }

    /**
     * Generiraj random password. Đerekova funkcija.
     * @return string
     */
    public static function generateRandomPassword()
    {
        //slova dozvoljena za password
        $allowed_chars = 'abcdefghijkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ';
        //znamenke dozvoljene za password
        $allowed_numbers = '23456789';

        $pwd_ch = str_shuffle($allowed_chars);
        $pwd_nu = str_shuffle($allowed_numbers);
        $pwd_all = str_shuffle($pwd_ch . $pwd_nu);
        //bar dva slova
        $pwd = substr($pwd_ch, rand(0, strlen($pwd_ch) - 1), 1);
        $pwd .= substr($pwd_ch, rand(0,strlen($pwd_ch) - 1), 1);
        //bar dvije znamenke
        $pwd .= substr($pwd_nu, rand(0, strlen($pwd_nu) - 1), 1);
        $pwd .= substr($pwd_nu, rand(0, strlen($pwd_nu) - 1), 1);
        // i jos 4 znaka
        for ($i=0; $i < 4; $i++) {
            $pwd .= substr($pwd_all, rand(0, strlen($pwd_all) - 1), 1);
        }
        // i jos malo pomjesati
        return str_shuffle($pwd);
    }

    /**
     * Generiraj datum do 31. listopada u trenutnoj ili idućoj godini.
     * @return string
     */
    public static function generateDefaultHrEduPersonExpireDate()
    {
        $currentDate = date('Y-m-d');
        $currentYear = intval(date('Y'));
        $octoberDate = date('Y-m-d', strtotime($currentYear . '-10-31'));
        $octoberDateYear = $currentYear;

        if($currentDate > $octoberDate) {
            $octoberDateYear = $currentYear + 1;
        }

        return date(static::$defaultDateFormat, strtotime($octoberDateYear . '-10-31'));
    }

    /**
     * Dohvati defaultni format datuma (tipično se koristi za UI)
     * @return string
     */
    public static function getDefaultDateFormat()
    {
        return static::$defaultDateFormat;
    }

    /**
     * Generiraj DN za danog korisnika
     * @param LdapUser $user
     * @return string
     */
    public static function generateDnForUser(LdapUser $user)
    {
        return static::generateDnForUid($user->uid[0]);
    }

    /**
     * Generiraj DN za uid
     * @param $uid
     * @return string
     */
    public static function generateDnForUid($uid)
    {
        return 'uid=' . $uid . ',' . static::getBaseDn();
    }

    /**
     * Generiraj ključ za cache koji će se koristiti za provjeru / dohvat vrijednosti
     * @param $uid
     * @return string
     */
    public static function generatePasswordResetCacheKey($uid)
    {
        return 'password-reset-' . $uid;
    }

    /**
     * Dohvati tipove admina
     * @return array
     */
    public static function getAdminTypes()
    {
        return static::$adminTypes;
    }

    /**
     * Izluči uid iz DN-a. Npr. iz 'uid=u100,dc=inter2,dc=hr' vrati 'u100'
     * @param $dn
     * @return mixed
     */
    public static function extractUidFromDn($dn)
    {
        return explode(',', substr($dn,4))[0];
    }

    /**
     * Keširaj informaciju o promijenjenoj zaporci za korisnika.
     * Koristi se za prikaz obavijesti o zaposrci
     * @param $uid
     * @param $password
     * @param int $duration
     * @throws \Exception
     */
    public static function cachePasswordResetInformation($uid, $password, $duration = 10)
    {
        $cacheKey = static::generatePasswordResetCacheKey($uid);

        $encryptedPassword = encrypt($password);

        cache([$cacheKey => $encryptedPassword], $duration);
    }

    /**
     * Generiraj URN string
     * @param string $key
     * @param array $entries
     * @return string
     */
    public static function generateUrn(string $key, array $entries)
    {
        if (empty($entries)) {
            return $key;
        }

        $urn = 'urn:' . $key;

        foreach ($entries as $attributeKey => $values) {
            foreach ($values as $attributeValue) {
                $urn .= ':' . $attributeKey . ':' . $attributeValue;
            }
        }

        return $urn;
    }

    /**
     * Provjeri dali među danim atributima postoji podatak o password policy
     * @param array $userSearchResult rezultat userSearch() metode iz AOSI Core-a.
     * @return bool
     * @throws \Exception
     */
    public static function isPasswordPolicyActive(array $userSearchResult)
    {
        if (! $userSearchResult) {
            throw new \Exception('Rezultat ne sadrži podatke za koje je moguće provjeriti password policy');
        }
        $attributes = head($userSearchResult);
        $passwordPolicyKey = self::PASSWORD_POLICY_INDICATOR_KEY;
        return isset($attributes[$passwordPolicyKey]) && !empty($attributes[$passwordPolicyKey]);
    }
}