<?php


namespace App\Helpers;


use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Illuminate\Validation\ValidationException;
use App\Helpers\HrEdu\HrEduPersonHelper;

class CsvHelper
{
    protected static $encodings = [
        'WINDOWS-1250' => 'Central european (windows 1250)',
        'ISO-8859-2' => 'Central european (ISO 8859-2)',
        'UTF-8' => 'UTF-8',
    ];

    public static function getEncodings()
    {
        return static::$encodings;
    }

    public static function validateCsvUploadRequest(Request $request, $noDelimiter = false)
    {
        $rules = [
            'csv_file' => 'required|file|mimetypes:text/plain|max:2048',
            'encoding' => [
                'required',
                Rule::in(array_keys(static::$encodings))
            ]
        ];
        $messages = [
            'csv_file.required' => 'Potrebno je odabrati tekstualnu datotektu',
            'csv_file.mimetypes' => 'Datoteka nije u dobrom formatu',
            'csv_file.max' => 'Datoteka je prevelika',
        ];

        if (!$noDelimiter) {
            $rules['delimiter'] = 'required|string|max:1';
            $messages['delimiter.required'] = 'Delimiter je obavezan';
        }

        Validator::make($request->all(), $rules, $messages)->validate();
    }

    /**
     * Pripremi podatke iz uploadane CSV datoteke
     * @param UploadedFile $file
     * @param $delimiter
     * @param $encoding
     * @param $neededAttributes Atributi koji moraju postojati u podacima
     * @param $baseDn
     * @param bool $shouldUnsetUid Dali maknuti uid iz podataka (npr. prilikom modifikacije korisnika, uid se ne smije moći mijenjati)
     * @return array
     * @throws ValidationException
     */
    public static function getCsvDataFromUploadedFile(
        UploadedFile $file,
        $delimiter,
        $encoding,
        $neededAttributes,
        $baseDn,
        $shouldUnsetUid = false
    )
    {
        $rawData = array_map(function ($row) use ($delimiter, $encoding) {
            return str_getcsv($row, $delimiter);
        }, static::getDataFromUploadedFile($file, $encoding));

        // Prvi red su ključevi atributa. Izvuci ih, lowercase-aj ih, i makni taj dio iz $rawData.
        $attributeKeys = array_map('strtolower', array_shift($rawData));

        // Provjeri dali su dani svi potrebni atributi
        $missingNeededAttributes = array_diff($neededAttributes, $attributeKeys);

        if ($missingNeededAttributes) {
            $message = 'Nisu dani potrebni atributi: ';
            $message .= implode(', ', $missingNeededAttributes);
            throw ValidationException::withMessages(['csv_file' => $message]);
        }

        $userHelper = new HrEduPersonHelper();

        $data = [];
        array_walk($rawData, function ($item) use ($attributeKeys, $baseDn, $userHelper, $shouldUnsetUid, &$data) {
            $keyedItem = array_combine($attributeKeys, $item);
            $dn = 'uid=' . $keyedItem['uid'] . ',' . $baseDn;

            // Kod modifikacije korisnika, uid se ne smije moći mijenjati, pa se može koristiti ovaj switch.
            if ($shouldUnsetUid) {
                unset($keyedItem['uid']);
            }

            $data[$dn] = $userHelper->normalizeModifiableAttributes($keyedItem);
        });

        return $data;
    }

    /**
     * Učitaj linije iz datoteke i dohvati ih kao array stringova, po potrebi enkodirajući stringove u UTF-8
     * @param UploadedFile $file
     * @param $encoding
     * @return array
     */
    public static function getDataFromUploadedFile(UploadedFile $file, $encoding)
    {
        // Encoding mora biti podržan
        if (!array_key_exists($encoding, static::getEncodings())) {
            return [];
        }

        return array_map(function ($row) use ($encoding) {
            // Trenutno ne koristi transliterate ili ignore...
            if ($encoding != 'UTF-8') {
                $row = iconv($encoding, 'UTF-8', $row);
            }
            return trim($row);
        }, file($file->getRealPath()));
    }
}