<?php


namespace App\Helpers\HrEdu;


use Illuminate\Http\Request;
use App\Helpers\AosiCoreHelper;

abstract class AbstractHrEduHelper
{
    /**
     * Instanca AOSI Core-a
     * @var \aosi\Aosi_core
     */
    protected $aosiCore;

    /**
     * Definicija atributa. Klase koje nasljeđuju ovu Abstract klasu  trebaju popuniti ovaj property.
     * Pogledaj HrEduPersonHelper ili HrEduOrgHelper
     * @var array
     */
    protected $attributeDefinitions;

    /**
     * Regex za provjeru dali je string prazan. Napravljeno ovako jer npr. empty('0') daje true.
     * @var string
     */
    protected $nonEmptyStringRegex = '/\S+$/i'; // Mora postojati baerem jedan non space znak.

    /**
     * Parent konstruktor kojeg trebaju pozvati child klase.
     * AbstractHrEduHelper constructor.
     */
    public function __construct()
    {
        $this->aosiCore = AosiCoreHelper::init();

        // Child metode trebaju definirati property $attributeDefinitions u svojim konstruktorima
    }

    /**
     * Dohvati hardkodirane ključeve atributa koje korisnk nikad nesmije moći mijenjati (override onog što je
     * definirano u definiciji atributa).
     * @return array
     */
    abstract public function getNonModifiableAttributes();

    /**
     * Dohvati ključeve atributa po tipu korisnika, akciji i obaveznosti. Pogledaj primjer HrEduPerson::get_allowed_attrs()
     * @param $userType
     * @param $action
     * @param $must
     * @return array
     */
    abstract public function getAllowedAttributes($userType, $action, $must = false);

    /**
     * Dohvati trenutne definicije atributa
     * @return array
     */
    public function getAttributeDefinitions()
    {
        return $this->attributeDefinitions;
    }

    /**
     * Vrati array ključeva atributa koji su tip binary
     * @param array $attributeKeys
     * @return array
     */
    public function filterBinaryAttributes(array $attributeKeys)
    {
        return array_filter($attributeKeys, function ($attributeKey) {
            return $this->attributeDefinitions[$attributeKey]->type == 'binary';
        });
    }

    /**
     * Dohvati ključeve binary atributa koje korisnik može modificirati
     * @param string $userType
     * @return array
     */
    public function getModifiableBinaryAttributes($userType)
    {
        $userChangeableAttributes = $this->getAllowedAttributes($userType, 'change');

        $binaryAttributes = $this->filterBinaryAttributes($userChangeableAttributes);

        return array_filter($userChangeableAttributes, function ($value) use ($binaryAttributes) {
            return in_array($value, $binaryAttributes);
        });
    }

    /**
     * Dohvati ključeve atributa koje korisnik može modificirati
     * @param string $userType user ili admin
     * @return array
     */
    public function getModifiableAttributes($userType)
    {
        // Onemogućeno modificiranje sljedećih atributa jer se hendlaju na poseban način.
        $nonModifiableAttributes = $this->getNonModifiableAttributes();

        $userChangeableAttributes = $this->getAllowedAttributes($userType, 'change');

        // Binarni atributi nisu za modifikaciju.
        // Npr. slika se može upload-ati, pogledati ili brisati pomoću svojih gumbića.
        $binaryAttributes = $this->filterBinaryAttributes($userChangeableAttributes);

        return array_filter($userChangeableAttributes, function ($value) use ($nonModifiableAttributes, $binaryAttributes) {
            return !in_array($value, $nonModifiableAttributes) && !in_array($value, $binaryAttributes);
        });
    }

    /**
     * Vrati vrijednosti iz arraya u odnosu na $shouldInvert. Ako je $shouldInvert false, vraća popunjene vrijednosti,
     * inače prazne vrijednosti.
     *
     * @param array $attributeRequestRawValues Request array za filtrirati
     * @param bool $shouldInvert dali napraviti invert rezultata (npr. umjesto popunjenih, vrati prazne vrijednosti)
     *
     * @return array
     */
    public function filterEmptyRawRequestValues(array $attributeRequestRawValues, $shouldInvert = false)
    {
        $attributesWithoutNullValues = $this->removeNullValuesFromRequestArray($attributeRequestRawValues);

        return array_filter($attributesWithoutNullValues, function ($value, $key) use ($shouldInvert) {
            // Mora postojati baerem jedan non space znak.
            $regex = $this->nonEmptyStringRegex;
            $shouldInclude = false;

            // Ako je array, uključi ako nije prazna (jer si prethodno već brisao prazne.
            if (is_array($value)) {
                if (!empty($value)) {
                    $shouldInclude = true;
                } else {
                    $shouldInclude = false;
                }
            } else {
                // Provjeri dali je vrijednost prazna.
                $shouldInclude = preg_match($regex, $value);
            }

            // $shoulInvert se koristi za dobivanje praznih atributa.
            if ($shouldInvert) {
                return !$shouldInclude;
            }

            return $shouldInclude;
        }, ARRAY_FILTER_USE_BOTH);
    }

    /**
     * Vrati prazne vrijednosti iz array-a u odnosu na obaveznost atributa u shemi.
     * @param array $attributeRequestRawValues
     * @param bool $shouldInvert
     * @return array
     */
    public function filterDeletableAttributes(array $attributeRequestRawValues, $shouldInvert = false)
    {
        // Brisanje se radi na način da se vrijednosti isprazne.
        // Dohvati sve atribute koji su prazni.
        $emptyAttributes = $this->filterEmptyRawRequestValues($attributeRequestRawValues, true);

        return array_filter($emptyAttributes, function ($key) use ($shouldInvert) {
            // Dohvati ključeve obaveznih atributa po shemi
            $mandatoryAttributes = $this->getAllowedAttributes('user', 'view', true);
            // Ako se koristi invert, onda vrati ako je na listi obaveznih (a prazan je). Smisao je dobiti listu
            // onih atributa koji su prazni (htjelo ih se brisati), a obavezni su po shemi.
            if ($shouldInvert) {
                return in_array($key, $mandatoryAttributes);
            }

            // Ako nije na listi obaveznih, onda je brisanje ok, pa vrati.
            return !in_array($key, $mandatoryAttributes);
        }, ARRAY_FILTER_USE_KEY);
    }

    /**
     * Filtriraj samo vrijednosti sa danim ključevima
     * @param $items
     * @param $allowedKeys
     * @return array
     */
    public function filterAllowedKeys($items, $allowedKeys)
    {
        if (!is_array($items) || !$allowedKeys) {
            return $items;
        }

        return array_filter($items, function ($key) use ($allowedKeys) {
            return in_array($key, $allowedKeys);
        }, ARRAY_FILTER_USE_KEY);
    }

    /**
     * Normaliziraj atribute namijenjene brisanju na način da ih pretvoriš u ['key' => []].
     * Taj format će brisati sve vrijednosti atributa, bez obzira dali su single ili multi value.
     * @param array $deletableAttributes
     * @return array
     */
    public function normalizeDeletableAttributes(array $deletableAttributes)
    {
        // Svaki key će imati prazan array, ovako ['key' => []]
        return array_fill_keys(array_keys($deletableAttributes), []);
    }

    /**
     * Normaliziraj array na način da je spremna za slanje za modifikaciju atributa.
     * @param array $modifiableAttributeValues
     * @return array
     */
    public function normalizeModifiableAttributes(array $modifiableAttributeValues)
    {
        // Ključevi atributa koji su tipa date
        $dateAttributes = $this->getDateAttributeKeys();
        array_walk($modifiableAttributeValues, function (&$value, $key) use ($dateAttributes) {
            // Uvijek vrati array za svaki atribut
            if (!is_array($value)) {
                $value = [$value];
            }

            // Reindeksiraj array kako bi indeski bili uvijek dobro poredani.
            $value = array_values($value);

            // -- Normalizacija --
            // Ako je datum...
            if (in_array($key, $dateAttributes)) {
                $value = array_map(function ($element) {
                    return formatDateString($element, 'hr', 'ldap');
                }, $value);
            }

            // Ako se radi o atributima sa prefiksima...
            if (array_key_exists($key, AosiCoreHelper::getPrefixedAttributes())) {
                // Mora biti string formatiran ovako: 'prefix: value'
                $newValue = [];
                foreach ($value as $unique) {
                    $uniquePrefix = trim($unique['prefix']);
                    $uniqueValue = trim($unique['value']);
                    if (!empty($uniquePrefix && !empty($uniqueValue))) {
                        $newValue[] = $uniquePrefix . ': ' . $uniqueValue;
                    }
                }

                $value = $newValue;
            }
        });

        return $modifiableAttributeValues;
    }

    /**
     * Vrati sve null vrijednosti unutar array kao empty string '', rekurzivno.
     * @param array $attributes
     * @return array
     */
    public function stringifyAttributeValues(array $attributes)
    {
        return array_map(function ($value) {
            if (is_array($value)) {
                return $this->stringifyAttributeValues($value);
            } else {
                return $value ?? '';
            }
        }, $attributes);
    }

    /**
     * Makni sve null vrijednosti iz array, rekurzivno.
     * @param array $attributes
     * @return array
     */
    public function removeNullValuesFromArrayRecursive(array $attributes)
    {
        // Osiguraj rekurzivno pozivanje funkcije...
        $array = array_map(function ($value) {
            if (is_array($value)) {
                return $this->removeNullValuesFromArrayRecursive($value);
            }
            return $value;
        }, $attributes);

        // Filtriraj trenutni array. Vrati ako je array ili ako nije barem jedan non space znak.
        $regex = $this->nonEmptyStringRegex;
        return array_filter($array, function($value) use ($regex) {
            return (is_array($value)) || preg_match($regex, $value);
        });
    }

    /**
     * Makni sve null vrijednosti iz request inputa koji su tipa array.
     * Ako nije array, onda vrati trenutnu vrijednot, makar bila null.
     * @param array $attributes
     * @return array
     */
    public function removeNullValuesFromRequestArray(array $attributes)
    {
        return array_map(function ($value) {
            if (is_array($value)) {
                return $this->removeNullValuesFromArrayRecursive($value);
            }

            return $value;
        }, $attributes);
    }

    /**
     * Izvuci raw content iz uploadanih datoteka
     * @param array $uploadedFiles
     * @return array
     */
    public function extractUploadedFilesContent(array $uploadedFiles)
    {
        $extractedContent = [];

        foreach($uploadedFiles as $attributeKey => $attributeValues) {
            array_walk($attributeValues, function (&$value) {
                $value = file_get_contents($value->getRealPath());
            });
            $extractedContent[$attributeKey] = $attributeValues;
        }

        return $extractedContent;
    }

    /**
     * Dohvati ključeve atributa koji su tipa date
     * @return array
     */
    public function getDateAttributeKeys()
    {
        return array_keys(array_filter($this->attributeDefinitions, function ($attributeDefinition) {
            return $attributeDefinition->type == 'date';
        }));
    }

    /**
     * Pripremi vrijednosti iz request-a tako da su spremni za slanje na modifikaciju
     * @param Request $request
     * @param $bindType
     * @return array
     */
    public function prepareModifiableAttributeValues(Request $request, $bindType)
    {
        // Lista ključeva za atribute koje korisnik smije modificirati
        $modifiableAttributes = $this->getModifiableAttributes($bindType);

        // Iz requesta dohvati samo one vrijednosti koje korisnik smije modificirati.
        $rawValues = $request->only($modifiableAttributes);

        // Makni sve prazne vrijednosti
        $values = $this->filterEmptyRawRequestValues($rawValues, false);

        // Obavi normalizaciju vrijednosti
        $values = $this->normalizeModifiableAttributes($values);

        return $values;
    }

    /**
     * Pripremi binarne vrijednosti iz requesta tako da su spremni za slanje na dodavanje
     * @param Request $request
     * @param $bindType
     * @return array
     */
    public function prepareModifiableBinaryAttributeValues(Request $request, $bindType)
    {
        $modifiableBinaryAttributes = $this->getModifiableBinaryAttributes($bindType);
        $rawValues = $request->only($modifiableBinaryAttributes);
        $values = $this->filterEmptyRawRequestValues($rawValues, false);

        // Ako ima kakvih vrijednosti, normaliziraj i extractaj sadržaj datoteke.
        if ($values) {
            $values = $this->normalizeModifiableAttributes($values);
            $values = $this->extractUploadedFilesContent($values);
        }

        return $values;
    }

    /**
     * Pripremi atribute iz requesta tako da se mogu slati na brisanje
     * @param Request $request
     * @param $bindType
     * @return array
     */
    public function prepareDeletableAttributeValues(Request $request, $bindType)
    {
        $modifiableAttributes = $this->getModifiableAttributes($bindType);
        $rawValues = $request->only($modifiableAttributes);

        // Ostavi samo one ključeve koji su na formi prethodno imali neku vrijednost
        $valuables = array_keys($request->get('valuables'));
        $rawValues = $this->filterAllowedKeys($rawValues, $valuables);

        $values = $this->filterDeletableAttributes($rawValues);

        if ($values) {
            $values = $this->normalizeDeletableAttributes($values);
        }

        return $values;
    }

    /**
     * Validiraj update request.
     * @param Request $request
     * @return
     */
    public function validateUpdateRequest(Request $request, $bindType)
    {
        // Po potrebi omogućiti validaciju i za ostale atribute...
        // Podsjetnik - validacija prisutnosti hrEduPersonUniqueNumber i hredupersonprimaryaffiliation = gost kada je oib = NONE
        // Trenutno samo datume...
        $dateAttributes = $this->getDateAttributeKeys();
        $attributeKeys = array_merge($dateAttributes);

        $rules = [];
        $messages = [];
        foreach ($attributeKeys as $attributeKey) {
            $attributeDefinition = $this->attributeDefinitions[$attributeKey];

            // Pravilo za signle element
            $singleRuleElements = [];
            // Pravilo za multiple element
            $multipleRuleElements = [];

            $requiredOrNullable = 'nullable';
            // Ako je atribut obavezan i trenutni bind type ga može mijenjati, onda je required.
            if ($attributeDefinition->must == 'true' && $attributeDefinition->change[$bindType] == 'true') {
                $requiredOrNullable = 'required';
                $messages[$attributeKey . '.required'] = $attributeKey . ' je obavezan';
            }
            $singleRuleElements[] = $requiredOrNullable;

            // Ako je multiple, moramo validirati i array i unutarnje vrijednosti.
            if ($attributeDefinition->multiple == 'true') {
                $multipleRuleElements[] = $requiredOrNullable;
                $singleRuleElements[] = 'array';
                if ($attributeDefinition->type == 'string') {
                    $multipleRuleElements[] = 'string';
                }
                if ($attributeDefinition->type == 'date') {
                    $multipleRuleElements[] = 'aosi_date';
                    $messages[$attributeKey . '.*.aosi_date'] = $attributeKey . ' nije u dobrom formatu';
                }
                if ($attributeDefinition->regex) {
                    $multipleRuleElements[] = 'regex:' . $attributeDefinition->regex;
                    $messages[$attributeKey . '.*.regex'] = $attributeKey . ' nije u dobrom formatu';
                }
                if (isset($attributeDefinition->attrtype['length'])) {
                    $multipleRuleElements[] = 'max:' . $attributeDefinition->attrtype['length'];
                    $messages[$attributeKey . '.*.max'] = $attributeKey . ' je predug';
                }
            } else {
                // Validiraj samo single vrijednost
                if ($attributeDefinition->type == 'string') {
                    $singleRuleElements[] = 'string';
                }
                if ($attributeDefinition->type == 'date') {
                    $singleRuleElements[] = 'aosi_date';
                    $messages[$attributeKey . '.aosi_date'] = $attributeKey . ' nije u dobrom formatu';
                }
                if ($attributeDefinition->regex) {
                    $singleRuleElements[] = 'regex:' . $attributeDefinition->regex;
                    $messages[$attributeKey . '.regex'] = $attributeKey . ' nije u dobrom formatu';
                }
                if (isset($attributeDefinition->attrtype['length'])) {
                    $singleRuleElements[] = 'max:' . $attributeDefinition->attrtype['length'];
                    $messages[$attributeKey . '.max'] = $attributeKey . ' je predug';
                }
            }

            $rules[$attributeKey] = $singleRuleElements;
            if ($multipleRuleElements) {
                $rules[$attributeKey . '.*'] = $multipleRuleElements;
            }
        }

        return $request->validate($rules, $messages);
    }
}