<?php

namespace App\Http\Controllers;

use aosi\HrEduPerson;
use App\Helpers\AosiCoreHelper;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class AdminController extends Controller
{
    /**
     * Ključevi atributa koji su potrebni za filtriranje popisa korisnika
     * @var array
     */
    protected $attributeKeys = ['uid', 'hredupersonoib', 'hredupersonuniquenumber', 'givenname', 'sn',
        'hredupersonprimaryaffiliation', 'hredupersonexpiredate'];

    protected $aosiCore, $attributeDefinitions, $viewableAttributes;

    protected $encodings = [
        'WINDOWS-1250' => 'Central european (windows 1250)',
        'ISO-8859-2' => 'Central european (ISO 8859-2)',
        'UTF-8' => 'UTF-8',
    ];

    public function __construct()
    {
        $this->aosiCore = AosiCoreHelper::init();
        $this->attributeDefinitions = HrEduPerson::$attribute;
        $this->viewableAttributes = HrEduPerson::get_allowed_attrs('admin', 'view');
    }

    /**
     * Prikazuje stranicu Popis korisnika
     */
    public function userListShow()
    {
        $attributeDefinitions = $this->attributeDefinitions;
        $attributeKeys = $this->attributeKeys;
        $encodings = $this->encodings;
        return view('admin.popisKorisnika', compact('attributeDefinitions', 'attributeKeys', 'encodings'));
    }

    /**
     * Prikazuje stranicu za upravljanje korisnicima
     */
    public function bulkManagementShow(Request $request)
    {
        $attributeDefinitions = $this->attributeDefinitions;
        $attributeKeys = $this->attributeKeys;
        $input = $request->all();
        $results = null;

        // Ako su upotrijebljeni parametri u GET requestu, tretiraj ih kao filter i dohvati popis korisnika iz LDAP imenika
        if ($input)
        {
            $neededAttributes = ['uid', 'givenname', 'sn', 'hredupersonuniquenumber', 'hredupersonprimaryaffiliation', 'hredupersonexpiredate'];
            $results = $this->fetchUsers($input, $neededAttributes);
        }

        return view('admin.upravljanjePopisomKorisnika', compact('results', 'attributeDefinitions', 'attributeKeys'));
    }

    /**
     * Vraća popis korisnika na AJAX zahtjev. Funkcija odgovara na AJAX upite iz DataTables plugina
     * konfiguriranog za server-side processing (paginaciju).
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     */
    public function getUserList(Request $request)
    {
        $input = $request->all();
        $currentPage = $input['page'];
        $size = $input['length'];
        // Ako je size == -1 ("∞" u izborniku za broj redaka u tablici), izbjegni paginaciju i zatraži kompletan popis
        if (($size == -1) && $input['lastTotal'])
            $size = $input['lastTotal'];

        $neededAttributes = ['uid', 'cn', 'hredupersonuniquenumber', 'hredupersonprimaryaffiliation', 'hredupersonexpiredate'];
        $results = $this->fetchUsers($input['filter'], $neededAttributes, $size, $currentPage);

        if ($results < 0) {
            return response()->json([
                'ldapError' => true,
                'result' => $this->aosiCore->result()
            ], 500);
        }

        $data = $this->normalizeUserListResult($results);
        $count = $this->aosiCore->result()['count'] ?? 0;

        return response()->json([
            'draw' => $input['draw'],
            'recordsFiltered' => $count,
            'recordsTotal' => $count,
            'data' => $data
        ]);
    }

    public function export(Request $request)
    {
//        dd($request->all());
        $input = $request->all();

        $encoding = $input['encoding'] && array_key_exists($input['encoding'], $this->encodings) ? $input['encoding'] : 'UTF-8';

        if (!$input['attributes'])
            return response("Nije odabran nijedan atribut.", 400);

        // Presjek skupa potrebnih atributa i skupa dozvoljenih atributa daje konačan skup s kojim smijemo raditi
        $attributeKeys = array_intersect($input['attributes'], $this->viewableAttributes);
        // Izbaci binary atribute
        foreach ($attributeKeys as $key => $attributeKey)
            if (hrEduPerson::$attribute[$attributeKey]->type == 'binary')
                unset($attributeKeys[$key]);

        $results = $this->fetchUsers($input['filter'], $input['attributes']);

        if(!$results)
        {
            return response("Pretraga ne pronalazi nijednog korisnika.", 404);
        }

        $delimiter = ';';
        $valueDelimiter = ',';
        $filename = 'export_user_list.csv';

        if (is_array($results) && count($results) && count($attributeKeys))
        {
            header ('Cache-Control: cache');
            header ('Content-Disposition: attachment; filename="' . $filename . '"');
            header ('Content-Type: application/octet-stream; name="' . $filename . '"');
            header ('Pragma: public');

            $output = 'dn';

            // Imena atributa u prvom retku
            foreach ($attributeKeys as $attributeKey)
                $output .= $delimiter . hrEduPerson::$attribute[$attributeKey]->ldapname;
            $output .= "\n";
            array_unshift($attributeKeys, 'dn'); // Gurni 'dn' na prvo mjesto

            foreach ($this->normalizeUserListResult($results) as $dn => $values)
            {
                $newLine = $dn;
                foreach ($attributeKeys as $key)
                {
                    if ($key == 'dn')
                        continue;
                    if (is_array($values[$key]))
                    {
                        foreach ($values[$key] as $k => $v)
                        {
                            if (hrEduPerson::$attribute[$key]->type == 'date')
                                $v[$k] = $v[$k] == 'NONE' ? 'NONE' : date('d.m.Y.', strtotime($v[$k]));
                            $v[$k] = str_replace(array("\n","\r", $delimiter, $valueDelimiter), array(" ", " ", " ", " "), $v[$k]);
                        }
                        $values[$key] = implode($valueDelimiter, $values[$key]);
                    }
                    else
                    {
                        if (hrEduPerson::$attribute[$key]->type == 'date')
                            $values[$key] = $values[$key] == 'NONE' ? 'NONE' : date('d.m.Y.', strtotime($values[$key]));
                        $values[$key] = str_replace(array("\n","\r", $delimiter, $valueDelimiter), array(" ", " ", " ", " "), $values[$key]);
                    }
                    $newLine .= $delimiter . $values[$key];
                }
                $output .= $newLine . "\n";
            }

            $output = $row = iconv('UTF-8', $encoding, $output);
            return $output;
        }

        return null;
    }

    /**
     * Dohvaća popis korisnika iz LDAP-a zajedno s traženim atributima (onima do kojih korisnik ima pristup) filtriran
     * pomoću danog skupa parametara
     * @param array $filter
     * @param array $attributes
     * @return array|bool
     * @throws \Exception
     */
    public function fetchUsers(array $filter, array $attributes, $size = 0, $page = 1)
    {
        // Ako filter hrEduPersonExpireDate odgovara razdoblju (dva datuma: "D1 to D2"), rastavi na početni i krajnji
        // datum i prilagodi za upis u LDAP filter
        if (isset($filter['hredupersonexpiredate'])) {
            $parts = explode(' to ', $filter['hredupersonexpiredate']);
            if ($parts[1]) {
                unset($filter['hredupersonexpiredate']);
                $filter['hredupersonexpiredate<'] = $parts[1];
                $filter['hredupersonexpiredate>'] = $parts[0];
            }
        }

        $filter = array_map(
            function($value) {
                // Zamijeni prazne sa '*'
                return $value ?: '*';
            },
            $filter
        );

        $ldapFilter = AosiCoreHelper::generateLdapFilter($filter);
        $user = Auth::user();

        // Presjek skupa potrebnih atributa i skupa dozvoljenih atributa daje konačan skup s kojim smijemo raditi
        $attributesToReturn = array_intersect($attributes, $this->viewableAttributes);

        // Ako je definiran parametar za paginaciju, upotrijebi funkciju ldapList()
        if ($size)
            $results = $this->aosiCore->ldapList(
                $user->uid[0],
                $user->password[0],
                $user->basedn[0],
                $ldapFilter,
                $attributesToReturn,
                $page,
                $size
            );
        else
            $results = $this->aosiCore->ldapSearch(
                $user->uid[0],
                $user->password[0],
                $user->basedn[0],
                $ldapFilter,
                $attributesToReturn
            );

        return $results;
    }

    /**
     * Normalizira array dobiven iz funkcije ldapList() u jednostavniji oblik potreban za plugin DataTables:
     * $array[][<dn>][<ime_atributa>][]=><vrijednost_atributa> --> $array[][<ime_atributa>]=><vrijednost_atributa> )
     * @param array $results
     * @return array
     */
    public function normalizeUserListResult($results)
    {
        $data = [];

        if ($results && is_array($results))
        {
            if (array_keys($results) === range(0, count($results) - 1))
                $sequential = true;
            else
                $sequential = false;

            foreach ($results as $dn => $result)
            {
                if ($sequential)
                    $attributes = array_shift($result);
                else
                    $attributes = $result;

                $normalized = [];
                foreach ($attributes as $key => $subArray)
                {
                    if (count($subArray) > 1)
                        $value = $subArray;
                    else
                        $value = $subArray[0] ?? '';
                    $normalized[$key] = $value;
                }
                $data[$dn] = $normalized;
            }
        }

        return $data;
    }

    /**
     * Obriši korisnika iz LDAP-a na AJAX zahtjev
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function deleteUser(Request $request)
    {
        $input = $request->all();
        $uids = $input['uids'];

        $aosiCore = AosiCoreHelper::init();
        $user = Auth::user();

        $baseDn = AosiCoreHelper::getBaseDn();
//        $a = is_array($uids);
//        return response()->json($input);
        foreach ($uids as $uid)
            $data[] = 'uid=' . $uid . ',' . $baseDn;

        /*return response()->json([
            'data' => $data
        ]);*/

        $deletionResult = $aosiCore->ldapDeleteUser(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $data
        );

        // Ako funkcija ldapDeleteUser() vrati negativan broj, vrati AJAX error i označi ga kao LDAP grešku,
        // kako bi znali prikazati odgovarajući opis u web-sučelju
        if (!$deletionResult) {
            return response()->json([
                'ldapError' => true,
                'result' => $aosiCore->result()
            ], 500);
        }

        return response()->json([
            'data' => $data
        ]);
    }

    /**
     * Promijeni datum isteka korisnicima na AJAX zahtjev
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function changeExpireDate(Request $request)
    {
        $input = $request->all();
        $uids = $input['uids'];
        $date = $input['date'];

        $aosiCore = AosiCoreHelper::init();
        $user = Auth::user();

        $baseDn = AosiCoreHelper::getBaseDn();
//        $a = is_array($uids);
//        return response()->json($input);
        foreach ($uids as $uid)
            $data['uid=' . $uid . ',' . $baseDn] = [
                'hredupersonexpiredate' => [
                    0 => $date
                ]
            ];

        $deletionResult = $aosiCore->ldapModifyAttribute(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $data
        );

        // Ako funkcija ldapDeleteUser() vrati negativan broj, vrati AJAX error i označi ga kao LDAP grešku,
        // kako bi znali prikazati odgovarajući opis u web-sučelju
        if (!$deletionResult) {
            return response()->json([
                'ldapError' => true,
                'result' => $aosiCore->result()
            ], 500);
        }

        return response()->json([
            'data' => $data
        ]);
    }
}
