<?php

namespace App\Http\Controllers\User;

use aosi\HrEduPerson;
use App\Extensions\LdapUser;
use App\Helpers\AosiCoreHelper;
use App\Helpers\HrEdu\HrEduPersonHelper;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;

class UserController extends Controller
{
    /**
     * Parametri za validaciju postavljanja nove lozinke.
     * @var array
     */
    protected $passwordParameters = [
        'newPassword' => [
            'pattern' => '(?=.*\d.*\d)(?=.*[A-Za-zšđčćžŠĐČĆŽ].*[A-Za-zšđčćžŠĐČĆŽ]).{8,}',
            'description' => 'Nova zaporka mora imati minimalno 8 znakova i sadržavati bar 2 znamenke i bar 2 slova',
        ],
        'oldPassword' => [
            'pattern' => '.{4,}',
            'description' => 'Nova zaporka mora imati minimalno 8 znakova i sadržavati bar 2 znamenke i bar 2 slova',
        ],
        'maxLength' => '128'
    ];

    public function edit()
    {
        $aosiCore = AosiCoreHelper::init();
        $userHelper = new HrEduPersonHelper();

        // Atributi koji će biti prikazani na formi
        $viewableAttributes = HrEduPerson::get_allowed_attrs('user', 'view');
        // Definicija svih postojećih atributa.
        $allAttributeDefinitions = $userHelper->getAttributeDefinitions();

        $user = Auth::user();

        $bindResult = $aosiCore->userSearch(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            '(uid=' . $user->uid[0] .')',
            $viewableAttributes
        );

        if ($bindResult) {
            $attributes = array_shift(array_values($bindResult));
            $ldapUser = new LdapUser($attributes);

            // Ako userSearch vrati result(), znači da korisnik mora promijeniti password.
            if (AosiCoreHelper::isPasswordPolicyActive($bindResult)) {
                AosiCoreHelper::setPasswordChangeRequirement(true);
                return redirect(route('user.password.edit'));
            }
        } else {
            throw new \UnexpectedValueException('Nije moguće pronaći podatke za ulogiranog korisnika.');
        }

        // Neka bude zahardkodirano da je view za user-a
        $bindType = 'user';
        $resource = $ldapUser;
        // Podesi rute, pošto se forma dijeli sa adminom.
        $formPostUrl = route('user.update');
        $pictureShowUrl = route('user.picture.show', ['hash' => '']);
        $pictureDeleteUrl = route('user.picture.delete', ['hash' => '']);

        return view('user.azuriranjePodataka')
            ->with(compact(
                'resource',
                'viewableAttributes',
                'allAttributeDefinitions',
                'bindType',
                'formPostUrl',
                'pictureShowUrl',
                'pictureDeleteUrl'
            ));
    }

    public function update(Request $request)
    {
        $aosiCore = AosiCoreHelper::init();
        $userHelper = new HrEduPersonHelper();

        $bindType = 'user';

        $userHelper->validateUpdateRequest($request, $bindType);

        $user = Auth::user();

        $userDn = AosiCoreHelper::generateDnForUser($user);

        $errorMessages = [];

        $normalizedModifiableAttributes = $userHelper->prepareModifiableAttributeValues($request, $bindType);

        if ($normalizedModifiableAttributes) {
            $data = [
                $userDn => $normalizedModifiableAttributes
            ];

            $modifyResult = $aosiCore->userModifyAttribute(
                $user->uid[0],
                $user->password[0],
                $user->basedn[0],
                $data
            );

            if (!$modifyResult) {
                $errorMessages[] = array_pop(array_values($aosiCore->result(true)))['message'];
            }
        }

        // Hendlanje dodavanja binaries
        $normalizedModifiableBinaryAttributes = $userHelper->prepareModifiableBinaryAttributeValues($request, $bindType);
//dd($normalizedModifiableBinaryAttributes);
        if ($normalizedModifiableBinaryAttributes) {

            $data = [
                $userDn => $normalizedModifiableBinaryAttributes
            ];

            $addResult = $aosiCore->userAddAttribute(
                $user->uid[0],
                $user->password[0],
                $user->basedn[0],
                $data
            );

            if (!$addResult) {
                $errorMessages[] = head($aosiCore->result(true))['message'];
            }
        }

        // Atributi koji su prazni, a koji nisu obavezni, mogu se brisati bez davanja vrijednosti.
        $deletableAttributes = $userHelper->prepareDeletableAttributeValues($request, $bindType);
        if ($deletableAttributes) {
            $data = [
                $userDn => $deletableAttributes
            ];

            $deleteResult = $aosiCore->userDeleteAttribute(
                $user->uid[0],
                $user->password[0],
                $user->basedn[0],
                $data
            );

            if (!$deleteResult) {
                $errorMessages[] = head($aosiCore->result())['message'] ?? 'Greška prilikom brisanja podataka';
            }
        }

        // Prikaži poruke o greškama prilikom svih operacija.
        if ($errorMessages) {
            flash('Došlo je do greške prilikom ažuriranja podataka.')->error();
            foreach ($errorMessages as $errorMessage) {
                flash($errorMessage)->warning()->important();
            }
        } else {
            flash('Podaci su ažurirani.')->success();
        }

        return redirect(route('user.edit'));
    }

    /**
     * Vrati sliku kroz HTTP
     * @param $hash
     * @return \Illuminate\Contracts\Routing\ResponseFactory|\Symfony\Component\HttpFoundation\Response
     */
    public function pictureShow($hash)
    {
        $aosiCore = AosiCoreHelper::init();
        $user = Auth::user();

        $binSearchResult = $aosiCore->userBinSearch(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            '(uid=' . $user->uid[0] .')',
            ['jpegphoto'],
            $hash
        );

        if (!$binSearchResult) {
            return response('', 404);
        }

        $image = head($binSearchResult)['jpegphoto'][0];
        return response(base64_decode($image), 200, ['Content-Type' => 'image/jpeg']);
    }

    /**
     * Obriši sliku
     * @param $hash
     * @return \Illuminate\Http\JsonResponse
     */
    public function pictureDelete($hash)
    {
        $aosiCore = AosiCoreHelper::init();

        $user = Auth::user();

        $userDn = AosiCoreHelper::generateDnForUser($user);

        $data = [
            $userDn => ['jpegphoto' => [$hash]]
        ];

        $deleteResult = $aosiCore->userDeleteAttribute(
            $user->uid[0],
            $user->password[0],
            $user->basedn[0],
            $data
        );

        if (!$deleteResult) {
            return response()->json([
                'result' => 'error',
                'message' => $aosiCore->err_message(),
                'code' => $aosiCore->err_code(),
                'pictureHash' => $hash
            ], 404);
        }

        return response()->json(['result' => 'ok']);
    }

    /**
     * Prikaži formu za promjenu zaporke
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function passwordEdit()
    {
        $passwordParameters = $this->passwordParameters;

        return view('user.promjenaZaporke', compact('passwordParameters'));
    }

    /**
     * Promijeni korisnikovu zaporku
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector
     */
    public function passwordUpdate(Request $request)
    {
        $input = $this->validatePasswordInput($request);

        $aosiCore = AosiCoreHelper::init();

        $user = Auth::user();

        // Koristimo trenutnu zaporku za bindanje na LDAP
        $oldPassword = base64_encode($input['old_password']);
        // Nova lozinka ide u čistom formatu, zbog validacije.
        $newPassword = $input['new_password'];

        $userDn = AosiCoreHelper::generateDnForUser($user);

        $data = [
            $userDn => [
                'userpassword' => [$newPassword]
            ]
        ];

        $modifyResult = $aosiCore->userModifyAttribute(
            $user->uid[0],
            $oldPassword,
            $user->basedn[0],
            $data
        );

        if (!$modifyResult) {
            flash($aosiCore->err_message())->error()->important();
            return back();
        }

        flash('Zaporka je promijenjena')->success();

        // Ažuriraj password u sesiji kako bi se mogao koristiti za naredne requeste prema LDAPu.
        $user->password = [base64_encode($newPassword)];
        Auth::setUser($user);

        // Makni naznaku da korisnik mora promijeniti password (ako je postojala).
        AosiCoreHelper::setPasswordChangeRequirement(false);

        return redirect(route('user.edit'));
    }

    /**
     * @param Request $request
     * @return mixed
     */
    protected function validatePasswordInput(Request $request)
    {
        return $request->validate([
            'old_password' => [
                'required',
                'string',
                'max:' . $this->passwordParameters['maxLength'],
                'regex:/' . $this->passwordParameters['oldPassword']['pattern'] . '/',
            ],
            'new_password' => [
                'required',
                'string',
                'max:' . $this->passwordParameters['maxLength'],
                'regex:/' . $this->passwordParameters['newPassword']['pattern'] . '/',
            ],
            'new_password_2' => [
                'required',
                'string',
                'same:new_password'
            ],
        ], [
            'old_password.required' => 'Stara zaporka je obavezna',
            'old_password.max' => 'Stara zaporka je preduga',
            'old_password.regex' => $this->passwordParameters['oldPassword']['description'],
            'new_password.required' => 'Nova zaporka je obavezna',
            'new_password.max' => 'Nova zaporka je preduga',
            'new_password.regex' => $this->passwordParameters['newPassword']['description'],
            'new_password_2.required' => 'Ponovite novu zaporku',
            'new_password_2.same' => 'Nova zaporka upisana u prvom i drugom polju se razlikuju.' .
                ' Molimo u oba polja upišite jednaku novu zaporku.',
        ]);
    }
}
